#==================================================
#TRABALHO IA367A - CIRCULINO
#Back-end
#==================================================


#======================================================================
# Pacote para manipulação de tabelas e vetores (função shift)
library(data.table) #Para uso da funcoes Shift, wday, hour as.ITime
library(dplyr) #rbind, #rbind_list -- para adicionar linhas no data frame
library(stringr) #str_length
library(lubridate) #USAR ESSE PARA Manipulação de datas (funcao floor_date e conversão de compo impordadas de csv com separadores de ";")

CAMPO_LAT=1
CAMPO_LONG=2
CAMPO_DATA=3
CAMPO_LINHA=4
CAMPO_CIRCULINO=5
CAMPO_VEL_MED=7  #velocidade calculada através das coordenadas (m/s)
CAMPO_HORARIO=6

#======================================================================
#PARÃMETROS DE SIMULAÇÃO ----
#Define qual o campo de velocidade a ser utilizado na avaliação
#CAMPO_VELOCIDADE=CAMPO_VEL_GPS  #Utilizar velocidade media medida (km/h) Obs.: Info não disponivel na base anual
CAMPO_VELOCIDADE=CAMPO_VEL_MED   #Velocidade calculadar pelas coordenadas
CAMPO_VELOCIDADE_CALC = 1       #Igno campos de velocidade e calcula a a partir das coordenadas GPS

#VAR_DS_PONTO
DIST_PONTO_APROX=25 #Variação de distância de aproximação da parada do Ônibus em relação ao ponto 
DIST_PONTO_AFAST=20 #14 #Variação de distância de afastamento da parada do Ônibus em relação ao local de parada (menor velocidade) 

VEL_MAX_PONTO_PARADA = 5 #Máxima velocidade do Ônibus que valida uma parada (m/s) 5*3,6 = 18 km/h
NUM_PONTOS_BUSCA = 4 #Ultimo ponto lido mais 3 pontos adiante

#======================================================================
#Função de cálculo de distância ----
DistGeodesica <- function(Lat1, Long1, Lat2, Long2) {
  
  #Retorna distância em metros entre pontos de duas coordenadas (Parâmetros de entrada em radianos)
  f <- (Lat1 + Lat2) / 2
  G <- (Lat1 - Lat2) / 2
  lambda = (Long1 - Long2) / 2
  s <- (sin(G) * cos(lambda)) ^ 2 + (cos(f) * sin(lambda)) ^ 2
  c <- (cos(G) * cos(lambda)) ^ 2 + (sin(f) * sin(lambda)) ^ 2
  omega <- atan(sqrt(s / c))
  r <- sqrt(s * c) / omega
  d <- omega * 2 * 6378120 #Multiplica pelo diâmetro da terra em metros
  H1 <- (3 * r - 1) / 2 / c
  H2 <- (3 * r + 1) / 2 / s
  Result <- d * (1 + H1 * (sin(f) * cos(G)/298257) ^ 2 - H2 * (cos(f) * sin(G)/298257) ^ 2)
  
  Result[which(is.na(Result))] <- 0  #Pontos com mesmas coordenadas resultam em NA --> Equivalem a valor ZERO
  
  return(Result)
}

#======================================================================
#ACESSO AOS DADOS DE RASTREAMENTO ----

TabRastreamentoTot <- read.csv(file="TabRastreamento.csv" ,header=T)
TabRastreamentoTot <- TabRastreamentoTot[,-1] #Elimina coluna de indice


#ACESSO AOS DADOS DE PONTOS DE ÃNIBUS
TabPontosTot <- read.csv(file="circular_pontos_por_linha.csv", header=T, sep=";", dec=",")



#======================================================================
#Inicio da Rotina principal ----

dfPonto <- c()

for(iLinha in 1:4) {
  
  dfParcial <- c()  #Data Frame com resultados particias
  
  TabPontos <- subset(TabPontosTot, linha==iLinha)

  TabPontosBusc <- c()
  TabPontosReduz <- data.frame(latitude=TabPontos$latitude * pi / 180,
                            longitude=TabPontos$longitude * pi / 180,
                            sequencia=TabPontos$sequencia)
  n = nrow(TabPontos)
  for (i in 1:n) {
    #Constroi vetores de busca de pontos
    j = i+1
    if (j>n) j=1
    if (i + NUM_PONTOS_BUSCA <= n+1) {
      #subvetor continuo
      TabPontosBusc[[j]] <- TabPontosReduz[i:(i + NUM_PONTOS_BUSCA-1),]
    }
    else {
      TabPontosBusc[[j]] <- bind_rows(TabPontosReduz [i:n,], TabPontosReduz [1:(NUM_PONTOS_BUSCA-(n-i+1)),])
    }
  }
  
  for(iCirculino in 1:8) { #Loop circulino ----
    print(paste("Linha=", as.character(iLinha),", Circulino=", as.character(iCirculino), sep="" ), quote=FALSE)
    
    #Seleciona o número do circulino de id da linha para análise
    TabRastreamento=subset(TabRastreamentoTot, id_circulino==iCirculino & id_linha==iLinha)
    TabRastreamento <- TabRastreamento[order(TabRastreamento$hora_coletado),]
    
    if (nrow(TabRastreamento)==0) next
    
    #===================================================
    # Remoção de linhas duplicadas (Mesmo horário de coleta)
    
    sDataAnterior <- ""
    iRastr=1
    RemoveLinhas=c()
    for(sData in TabRastreamento[[CAMPO_DATA]]) {
      if (sData == sDataAnterior) {
        RemoveLinhas=c(RemoveLinhas,-iRastr) #Adiciona linha com valor negativo para exclusao da tabela de rastreamento
      }
      iRastr<-iRastr+1
      sDataAnterior=sData
    }
    if (length(RemoveLinhas) > 0 )
      TabRastreamento <- TabRastreamento[RemoveLinhas,] #Remove linhas duplicadas
    
    #Converte graus para radianos
    LatiRad=TabRastreamento[[CAMPO_LAT]] * pi / 180  #Latitude 
    LongRad=TabRastreamento[[CAMPO_LONG]] * pi / 180  #Longitude
    
    
    #Conversão de texto em data
    
    #dt1=dmy_hms(TabRastreamento[1,CAMPO_DATA]) #Opção com lubridate (Necessita formatos predefinidos --> não adequado)
    #dt2=dmy_hms(TabRastreamento[2,CAMPO_DATA])
    
    #data=strptime(TabRastreamento[[CAMPO_DATA]], "%d/%m/%Y %H:%M:%S") #BR
    data=strptime(TabRastreamento[[CAMPO_DATA]], "%Y-%m-%d %H:%M:%S")
    #dataS=shift(as.numeric(data))
    #Desloca uma posição para o cáluclo de data para cálculo de intervalo entre mediÃÂ§ÃÂµes
    #dataS=strptime(shift(TabRastreamento[[CAMPO_DATA]]), "%Y-%m-%d %H:%M:%S") #Shift não funciona com datas (aplicar primeiro ao vetor string)
    
    DeltaTN <- as.numeric(data) - shift(as.numeric(data)) #dataS   #Intevalo entre medicoes 
    DeltaTN[1] <- 0
    
    de#------------------------------------------------------------------------------------------
    #Calcula distancia percorrida entre duas medicoes
    DeltaS <- DistGeodesica(LatiRad, LongRad, shift(LatiRad, fill=LatiRad[1]), shift(LongRad, fill=LongRad[1]) )
    
    #Converte velocidade km/h --> m/s
    #if(CAMPO_VELOCIDADE==CAMPO_VEL_GPS) TabRastreamento[[CAMPO_VEL_GPS]]=TabRastreamento[[CAMPO_VEL_GPS]]/3.6
    
    if (CAMPO_VELOCIDADE_CALC == 1) {
      #Velocidade não especifica --> necessário calcular velocidade a partir das coordenadas
      TabRastreamento[[CAMPO_VELOCIDADE]] <- DeltaS / DeltaTN
      TabRastreamento[1,CAMPO_VELOCIDADE] <- TabRastreamento[2,CAMPO_VELOCIDADE]  #Primeira posição é indefinida, assume valor susequente
    }
    #Calcula aceleração
    Aceleracao <- (TabRastreamento[[CAMPO_VELOCIDADE]] - shift(TabRastreamento[[CAMPO_VELOCIDADE]])) / DeltaTN
    nRastr <- nrow(TabRastreamento)
    
    #---------------------------------------------------------------------------------
    #Buscar na tabela de rastreamento todas as parados do ônibus (com velocidade e aceleração nulas) próxima ao ponto de
    #parada definda por LAT_PONTO e LONG_PONTO
    
    
    #Define vetores para armazenamento dos resultados nas coletados nas paradas de onibus
    PontoNum <- 0
    PontoCod <- 0
    PontoLocal <- 0
    PontoAceleracao <- 0
    PontoAcelIdx    <- 0
    #PontoHorarioViagem <- ""
    PontoData <- data[1]
    PontoDistIni <-0
    PontoDistParada <- 0
    PontoDistFim <- 0
    PontoVelParada <- 0
    PontoTempoParada <- 0
    PontoLat <- 0
    PontoLong <- 0
    
    PontoTrajetoria <- c(0)
    PontoTrajetoriaMinDist <- c(0)
    PontoTrajetoriaData <- c(0)
    PontoTrajetoriaIdx <- 1
    PontoTrajetoriaVerif <- 0
    
    Entorno <- 0
    iRastr <- 1
    EntornoIniIdx <- iRastr
    iEntorno <- 0
    RemovePontos <- c()
    sListPontos = ""
    
    iPonto <- 1  #Valor para iniciar busca do primeiro ponto
    
    PontosBusc <- TabPontosReduz #Na primeira iteracao procura por qualquer dos pontos de onibu

    repeat {
      # Varredura do rastreamento
      # 1-Identifica proximidade do ponto de parada
      # 2-Localizada ponto de imobilizacao (velocidade =0)
      # 3-Identifica retomada de velocidade
      # 4-Encontra maior aceleracao de retomada
      # 5-Validade quantidade minima de amostras para validar o resultado
      
      if (Entorno==0) {
        #Posicao distante do ponto de parada
        #Result <- ProxPonto () #Testa proximidade do proximo (Verifica ponto anterior para teste de falsa deteccao)
        iPonto = 0
        Dist <- DistGeodesica(LatiRad[iRastr], LongRad[iRastr], PontosBusc$latitude, PontosBusc$longitude)
        for (i in 1:nrow(PontosBusc)) {
          if (Dist[i] <= DIST_PONTO_APROX) {
            #Localizada a proximidade de um ponto de Ônibus
            EntornoD <- Dist[i]
            iPonto <- PontosBusc$sequencia[i]
            break
          }
        }

        if (iPonto > 0) { 
          sListPontos = paste(sListPontos, iPonto, sep=", ")
          
          PontosBusc <- TabPontosBusc[[iPonto]] #Pontos a serem buscado na próxima iteracao

          if (PontoTrajetoriaVerif > 0) { 
            #Ponto fora de sequencia detectado na iteracao anterior
            if (iPonto == PontoTrajetoriaVerif) {
              #Apareceu o ponto a ser verificado com possível deteccao incorreta
              if (iPonto == PontoTrajetoria[PontoTrajetoriaIdx-1]) {
                #----2Falso->---->--------1
                #                         |
                #----2----------<----------   Se leitura for 2, 1, 2 --> Primero 2 é leitura falsa
                #Verifica necessidade de romocao de medidas no ponto j falso
                if (iEntorno > 0) {
                  if (PontoNum[iEntorno]==iPonto)
                    #Adiciona ponto com valor negativo para exclusao do data frame final
                    RemovePontos<-c(RemovePontos,-iEntorno)
                  else if (iEntorno > 1) {
                    if (PontoNum[iEntorno-1]==iPonto)
                      #Adiciona ponto com valor negativo para exclusao do data frame final
                      RemovePontos<-c(RemovePontos,-(iEntorno-1))
                  }
                }
              }
            }
            PontoTrajetoriaVerif = 0
          }    
          else {
            #Sem verficao solcitada na iteracao anterior
            # n, 1, 2
            if (iPonto < PontoTrajetoria[PontoTrajetoriaIdx]) {
              #Verifica se passou do ponto final para o ponto inicial da linha --> Se sim, sequencia normal
              if (PontoTrajetoria[PontoTrajetoriaIdx] - iPonto < NUM_PONTOS_BUSCA) {
                #Ponto fora de sequencia (possivel interferencia de proximidade de pontosn na trajetoria)
                if (PontoTrajetoriaData[PontoTrajetoriaIdx] - PontoTrajetoriaData[PontoTrajetoriaIdx-1] < 900) {
                  #Diferenca pequena de tempo (<15 minutos)
                  if (PontoTrajetoriaIdx >= 2) {
                    if (iPonto == PontoTrajetoria[PontoTrajetoriaIdx-1]) {
                      #Ponto já já foi detectado na iteracao que antecedeu a anterior
                      #----1------>---->--------2
                      #                         |
                      #----1Falso-----<----------   Se leitura for 1, 2 e 1 --> Ultimo 1 é leitura falsa
                      #Aguarda saída da da proxidade da distancia de inicio do ultimo Entorno
                      Entorno = 4
                      PontoTrajetoriaData   [PontoTrajetoriaIdx+1] <- as.numeric(data[iRastr])
                      PontoTrajetoriaMinDist[PontoTrajetoriaIdx+1] <- 10000 #Valor elevado para desqualificar
                      iRastr <- iRastr+1
                      if (iRastr+1 > nRastr) break
                      EntornoDProx <- DistGeodesica(LatiRad[iRastr], LongRad[iRastr], TabPontosReduz$latitude[iPonto], TabPontosReduz$longitude[iPonto])
                    }
                    else {
                      #Ponto nao detectado anteriormente --> aguarda proxima iteracao para teste
                      PontoTrajetoriaVerif = iPonto # Registra para verificacao na proxima iteracao
                      #----2Falso->---->--------1
                      #                         |
                      #----2----------<----------   Se leitura for 2, 1, 2 --> Primero 2 sera leitura falsa
                    }
                  }
                }
              }
            }
          }
          
          
          
          PontoTrajetoriaIdx <- PontoTrajetoriaIdx + 1
          PontoTrajetoria[PontoTrajetoriaIdx] <- iPonto #Registra passagem pelo ponto
          
          if (Entorno == 0) { # Estado 0 - de busca por pontos ----
            EntornoIniIdx = iRastr #Posicao inicial do trecho no entorno da parada
            EntornoParadaIdx = iRastr #Posiciona indice de velocidade minima
            EntornoV = TabRastreamento[iRastr,CAMPO_VELOCIDADE] #Registra velocidade
            
            EntornoDistMin = EntornoD
            EntornoDistMinIdx = iRastr
            
            EntornoParadaIdx = iRastr #Registra posição da última ocorrência da menor velocidade
            EntornoParadaDist = EntornoD
            
            Entorno= 1
            
            iRastr <- iRastr+1
            if (iRastr+1 > nRastr) break
            EntornoDProx <- DistGeodesica(LatiRad[iRastr], LongRad[iRastr], TabPontosReduz$latitude[iPonto], TabPontosReduz$longitude[iPonto])
            
            if (EntornoD > EntornoDProx)
              #Aproximação do ponto --> Distância inicial negativa em relação ao ponto
              EntornoDistIni = -EntornoD
            else
              #Afastamento do ponto --> Distância inicial positiva em relação ao ponto
              EntornoDistIni = EntornoD
          }
        }
        else {
          #Prepara proxima iteracao
          iRastr = iRastr+1
          if (iRastr > nRastr) break
          #EntornoDProx <- DistGeodesica(LatiRad[iRastr], LongRad[iRastr], TabPontosReduz$latitude[iPonto], TabPontosReduz$longitude[iPonto])
        }
      }
      else {
        EntornoD = EntornoDProx #Distancia atual assume valor da distancia calculada no final da iteracao anterior
        
        if (Entorno==1) {
          #Aproximacao do ponto de parada --> Encontra a menor velocidade (igual ou proxima a ZERO)
          if (EntornoD <= DIST_PONTO_APROX) {
            #Procura posicionamento com velocidade nula
            EntornoVTmp = TabRastreamento[iRastr,CAMPO_VELOCIDADE]
            if (EntornoVTmp < EntornoV) {
              #Onibus paradado --> Aguarda aceleração e retomada de velocidade
              EntornoV = EntornoVTmp
              EntornoParadaIdx = iRastr #Registra posição da última ocorrência da menor velocidade
              EntornoParadaDist = EntornoD
            }
            if (EntornoD < EntornoDistMin) {
              EntornoDistMin = EntornoD
              EntornoDistMinIdx = iRastr
            }
          }
          else {
            #Final da analise de velocidade na proximidade do ponto
            PontoTrajetoriaMinDist[PontoTrajetoriaIdx]= EntornoDistMin
            PontoTrajetoriaData   [PontoTrajetoriaIdx]= as.numeric(data[EntornoDistMinIdx])
            if (PontoTrajetoriaIdx > 0) {
              #Verifica se houve repeticao do ponto de onibus
              if (iPonto == PontoTrajetoria[PontoTrajetoriaIdx-1]) {
                #Ponto atual é o mesmo que o ponto anteriormente detectado
                if (PontoTrajetoriaData[PontoTrajetoriaIdx] - PontoTrajetoriaData[PontoTrajetoriaIdx-1] < 900) { # 15 minutos
                  #Diferenca pequena de tempo (<15 minutos)
                  if (PontoTrajetoriaMinDist[PontoTrajetoriaIdx-1] < EntornoDistMin) {
                    #Deteccao anterior foi mais proxima do ponto --> abondona analise atual
                    Entorno = 4
                  }
                  else {
                    #Detecao atual está mais proxima do ponto
                    if (iEntorno >0) {
                      if (PontoNum[iEntorno] == iPonto) {
                        #Na iterecao anterior foi registrada uma parada nesse mesmo ponto
                        if (PontoTrajetoriaData[PontoTrajetoriaIdx] - as.numeric(PontoData[iEntorno]) < 900) { #15 minutos
                          #Ponto registrado é recente
                          iEntorno = iEntorno-1 #Remove registro
                        }
                      }
                    }
                  }
                }
              }
            }
            if (Entorno == 1) { # Estado 1 - Desacelera ----
              Entorno = Entorno+1 #0
              iRastr=EntornoParadaIdx #Retorna para a posicao de menor velocidade

              if (EntornoParadaIdx < EntornoDistMinIdx)
                #Aproximação do ponto --> Distância negativa em relacao a parada
                EntornoParadaDist = -EntornoParadaDist
              # else
              #   #Afastamento do ponto --> Distancia postiva em relacao a parada
              #   EntornoParadaDist = EntornoParadaDist
            }
          }
        }
        else if (Entorno==2) { # Estado 2 - Parado
          #Onibus parado --> Aguarda retomada de velocidade
          if (EntornoD - EntornoParadaDist <= DIST_PONTO_AFAST) {
            if (Aceleracao[iRastr]>0) {
              #Onibus acelera e retoma velocidade
              EntornoA = Aceleracao[iRastr] #Salva aceleracao de saida
              #EntornoAclive=TabRastreamento$altitude[iRastr]-TabRastreamento$altitude[iRastr-1]
              EntornoRetomadaIdx = iRastr
              EntornoRetomadaMax = iRastr
              EntornoRetomadaD = EntornoD
              EntornoFimD = 0
              Entorno= Entorno + 1
            }
          }
          else {
            Entorno = 0 #Abondona Entorno do ponto --> Dados insuficientes para calcular aceleração
          }
        }
        else if (Entorno==3) { # Estado 3 - Aceleração ----
          #Afastamento do ponto de parada --> fase de aceleração
          if (EntornoD - EntornoParadaDist <= DIST_PONTO_AFAST) {
            # ônibus próximo ao ponto de parada
            if (EntornoA < Aceleracao[iRastr]) {
              EntornoA = Aceleracao[iRastr] #Salva maior aceleração
              #EntornoAclive=TabRastreamento$altitude[iRastr]-TabRastreamento$altitude[iRastr-1]
              EntornoRetomadaMax = iRastr
            }
            EntornoFimD = EntornoD #Atualiza distância máxima de afastamento do ponto
          }
          else {
            # ônibus se afastou do ponto de parada acima do limite
            EntornoFimIdx = iRastr-1
            #Copia trajeto no entorno do ponto
            EntornoTempAcel = as.numeric(data[EntornoFimIdx]) - as.numeric(data[EntornoRetomadaIdx])
            if (EntornoTempAcel > 0 && 
                EntornoFimD - EntornoRetomadaD > 0 &&
                EntornoV <= VEL_MAX_PONTO_PARADA) {

              #Onibus chegou a acelerar com tempo, distancia percorrida suficiente para validar a medida, reduziu a velocidade para parada
              #Salva registro da parada ----
              iEntorno = iEntorno+1
              #print (paste("REGISTRO Num Ponto:", iPonto, "iEntorno:", iEntorno))
              PontoNum [iEntorno] = iPonto
              PontoCod [iEntorno] = TabPontos$codigo[iPonto]
              PontoLocal [iEntorno] = TabPontos$codigo.local[iPonto]
              PontoAceleracao[iEntorno] = EntornoA
              PontoAcelIdx[iEntorno] = as.integer(EntornoRetomadaMax)
              PontoData[iEntorno] = data[EntornoParadaIdx]
              PontoTempoParada[iEntorno] = as.numeric(data[EntornoRetomadaIdx]) - as.numeric(data[EntornoParadaIdx])
              
              PontoDistIni[iEntorno] = EntornoDistIni
              PontoDistParada[iEntorno] = EntornoParadaDist
              PontoDistFim[iEntorno] = EntornoFimD
              PontoVelParada[iEntorno] = EntornoV
              PontoLat[iEntorno] = TabRastreamento$latitude[EntornoParadaIdx]
              PontoLong[iEntorno] = TabRastreamento$longitude[EntornoParadaIdx]
              
              # tmp=as.character(TabRastreamento[EntornoFimIdx, CAMPO_HORARIO] )
              # if(str_length(tmp)==0)
              #   PontoHorarioViagem[iEntorno] = "Indef"
              # else
              #   PontoHorarioViagem[iEntorno] = substr(tmp, 1, 5)
            }
            if (EntornoD <= DIST_PONTO_APROX)
              #Inclui estado para garantir que a distância de fim não esteja dentro da distância de inicio de um Entorno
              Entorno = Entorno+1
            else
              Entorno = 0 #Retorna a busca para o proximo ponto
          }
        }
        else if (Entorno==4) {
          #Aguarda saída da proxidade da distancia de inicio do ultimo Entorno
          if (EntornoD > DIST_PONTO_APROX)
            Entorno = 0 #Retorna a busca para o proximo ponto
        }
        
        iRastr = iRastr+1
        if (iRastr > nRastr) break
        EntornoDProx <- DistGeodesica(LatiRad[iRastr], LongRad[iRastr], TabPontosReduz$latitude[iPonto], TabPontosReduz$longitude[iPonto])
      }
      
      if (as.numeric(data[iRastr]) - as.numeric(data[EntornoIniIdx]) > 600) { # 10 minutos de perda de dados desde o Ãºltimo ponto detectado
        EntornoIniIdx=iRastr
      #if(DeltaTN [iRastr] > 300) { #Mais que 5 minutos
        #Perda de dados --> Próxima iteração busca por todos os pontos
        PontosBusc <- TabPontosReduz
        print (paste("L", iLinha, " C", iCirculino, " Pontos em sequencia: ", sListPontos, sep=""), quote=FALSE)
        sListPontos=""
      }
    } #Termino da varredura por pontos de parada
    
    if (iEntorno > 0) { #(!is.null(PontoAceleracao)) {
      #Encontrados dados para adicionar ao data frame
      #Salva registros do circulino processado no data frame dfParcial ----
      dfTemp <- data.frame(Data=PontoData, TempoParada=PontoTempoParada, #Viagem=PontoHorarioViagem, 
                           Aceleracao=PontoAceleracao, CodPonto=PontoCod, CodLocal=PontoLocal,
                           NumPonto=PontoNum, Circulino = as.integer(iCirculino), AcelIdx=PontoAcelIdx,
                           DistIni=PontoDistIni, DistParada=PontoDistParada, DistFim=PontoDistFim,
                           VelParada=PontoVelParada,
                           Latitude=PontoLat, Longitude=PontoLong
      )
      #fTemp$Ponto <- as.character(TabPontos[PontoNum])
      #dfTemp$Viagem <- as.character(dfTemp$Viagem)
      
      if (!is.null(RemovePontos)) {
        dfTemp <- dfTemp[RemovePontos,]
      }
      if (is.null(dfParcial))
        dfParcial <- dfTemp
      else
        dfParcial <- bind_rows(dfParcial, dfTemp)
    }
    
    #} #Término do loop dos pontos de ônibus
    
  } #Loop iCirculino
  
  if (!is.null(dfParcial)) {
    #Salva registros da Linha processado no data frame dfPonto ----
    dfParcial$Linha <- as.integer(iLinha)
    
    if (is.null(dfPonto))
      #Data frame vazio
      dfPonto <- dfParcial
    else
      #Acrescenta dados ao final
      dfPonto <- bind_rows(dfPonto, dfParcial)
  }
  
} #Loop iLinha 

TabRastreamento <- c()  #Limpa
dfParcial <- c()
dfTemp <- c()
DeltaTN <- c()
DeltaS <-c()
Aceleracao <- c()

print("Processamento finalizado!", quote=FALSE)
print(paste("Total registros processado salvos: ", nrow(dfPonto)), quote=FALSE)

#Salva dados em Data Frame
# dfPonto$Horario=as.ITime(dfPonto$Data)
# dfPonto$Semana=wday(dfPonto$Data)
# dfPonto$Horario15<-as.ITime(floor_date(dfPonto$Data, "15 mins")) #Aproxima para o horário mais cedo (outra opção round_date)
# dfPonto$Horario15 <- factor(substr(dfPonto$Horario15, 1, 5))
# dfPonto$Mes=month(dfPonto$Data)
#dfPonto$Ponto=factor(dfPonto$Ponto)

write.csv(dfPonto, file="TabGrandFisicas.csv")
dfponto<-c()


